﻿(function () {
    "use strict";

    var canvas, context, stage, screen_width, screen_height, Monsters, contentManager;

    var spritesNumber, displayBench, currentWorkBench, dynamicSpritesNumberLabel;
    var shadowsEnabled = false;
    var useRAF = false;
    var dynamicBench = false;
    var dynMonster = 1;
    var dynJustAdded = false;
    var dynJustRemoved = false;
    var dynCounterAdd = 0;
    var dynCounterRemove = 0;
    var dynTargetFPS = 60;
    var dynTargetFPSArrayValue = [];

    var init = function (workbench, spritesNumParam, shadowsEnabledParam, canvasSizeXParam, canvasSizeYParam, then) {
        spritesNumber = spritesNumParam;
        shadowsEnabled = shadowsEnabledParam;
        displayBench = then;
        currentWorkBench = workbench;
        // Getting canvas and context
        canvas = document.createElement('canvas');
        workbench.appendChild(canvas);
        context = canvas.getContext("2d");

        screen_width = canvasSizeXParam;
        screen_height = canvasSizeYParam;

        // Setting hardware scaling
        canvas.width = screen_width;
        //canvas.style.width = '100%';
        canvas.height = screen_height;
        //canvas.style.height = '100%';
        canvas.style.zIndex = 0;

        // create a new stage and point it at our canvas:
        stage = new createjs.Stage(canvas);

        // Position
        canvas.style.position = "absolute";
        canvas.style.left = (workbench.clientWidth - screen_width) / 2 + "px";
        canvas.style.top = (workbench.clientHeight - screen_height) / 2 + "px";
        canvas.style.backgroundColor = "#607559";

        contentManager = new ContentManager();
        if (spritesNumParam < 0) {
            if (spritesNumber === -1) dynTargetFPS = 60;
            else dynTargetFPS = 30;
            spritesNumber = 0;
            dynamicBench = true;

            dynamicSpritesNumberLabel = document.createElement("div");
            dynamicSpritesNumberLabel.style.backgroundColor = "white";
            dynamicSpritesNumberLabel.style.width = "100px";
            dynamicSpritesNumberLabel.style.height = "20px";
            dynamicSpritesNumberLabel.style.textAlign = "center";
            workbench.appendChild(dynamicSpritesNumberLabel);
        }
        contentManager.SetDownloadCompleted(buildFixNumberOfSprites);
        contentManager.StartDownload();
    };

    var buildFixNumberOfSprites = function () {
        // Our Monsters collection
        Monsters = new Array();

        for (var xMonsters = 0; xMonsters < spritesNumber; xMonsters++) {
            var newRandomMonster = new Monster(contentManager, shadowsEnabled, screen_width, screen_height);
            Monsters.push(newRandomMonster);
            stage.addChild(newRandomMonster);
        }

        // we want to do some work before we update the canvas,
        // otherwise we could use Ticker.addListener(stage);
        createjs.Ticker.addEventListener("tick", tick);
        // Best Framerate targeted (60 FPS)
        createjs.Ticker.useRAF = useRAF;
        createjs.Ticker.setFPS(60);

        // Waiting 1s before doing taking first FPS due to some warm up needed on most machines
        setTimeout(displayBench, 1000);
    };

    // Function that adds/removes sprites to maintain 30 or 60 FPS
    var dynamicSpritesToStage = function () {
        var currentFPS = POTATOES.GamingBench.currentFPS;
        // Current number of sprites drawn on screen
        var currentLength = Monsters.length;
        dynamicSpritesNumberLabel.innerHTML = currentLength;
        // Keeping an history of current "optimum" number of sprites
        dynTargetFPSArrayValue.push(currentLength);
        // If the FPS is above target, there is a potential to add some more
        if (currentFPS >= dynTargetFPS - 1) {
            // We're waiting for a several 60 or 30 fps targets before trying to add more
            // Otherwise, the measured FPS is too volatile 
            dynCounterAdd++;
            if (dynCounterAdd > 15) {
                for (var xMonsters = 0; xMonsters < dynMonster; xMonsters++) {
                    var newRandomMonster = new Monster(contentManager, shadowsEnabled, screen_width, screen_height);
                    Monsters.push(newRandomMonster);
                    stage.addChild(newRandomMonster);
                }
                // If we've just removed some sprites, we need to adjust the value
                // to add less sprites to avoid re-falling directly into the same adjustment
                if (dynJustRemoved) dynMonster = dynMonster / 2;
                // Otherwise, we're just adding more and more monsters
                else dynMonster *= 2;
                dynJustAdded = true;
                dynJustRemoved = false;
                dynCounterAdd = 0;
            }
        }
        else {
            // if we're above the target, we probably have too many sprites on screen
            // Please note that we have 
            if (currentFPS < dynTargetFPS - 2) {
                dynCounterRemove++;
                if (dynCounterRemove > 30) {
                    if (dynJustAdded) dynMonster = dynMonster / 2;
                    else dynMonster++;
                    for (var i = 0; i < dynMonster; i++) {
                        if (currentLength > 0) {
                            stage.removeChildAt(currentLength);
                            Monsters.pop();
                            currentLength--;
                        }
                        else {
                            break;
                        }
                    }

                    dynJustAdded = false;
                    dynJustRemoved = true;
                    dynCounterRemove = 0;
                }
            }
        }
    };

    // tick function called-back by EaselJS
    var tick = function () {
        if (dynamicBench) dynamicSpritesToStage();

        // looping inside the Monsters collection
        for (var monster in Monsters) {
            var m = Monsters[monster];
            // Calling explicitly each tick method 
            // to launch the update logic of each monster
            m.tick();
        }

        // update the stage:
        stage.update();
    }

    var stopEaselJSBench = function () {
        stage.removeAllChildren();
        createjs.Ticker.removeAllListeners();
        stage.update();
        currentWorkBench.removeChild(canvas);
    }

    var easelJSSpritesBench_200_NoShadow_640_480 = new POTATOES.GamingBench.Bench("Drawing 200 sprites (640x480)", "http://blogs.msdn.com/davrous",
        function (workbench) { // Init
            init(workbench, 200, false, 640, 480, this.onInitCompleted);
        }, function () { }, function (workbench) { // End
            stopEaselJSBench();
        });

    var easelJSSpritesBench_200_Shadows_640_480 = new POTATOES.GamingBench.Bench("Drawing 200 sprites with shadows (640x480)", "http://blogs.msdn.com/davrous",
        function (workbench) { // Init
            init(workbench, 200, true, 640, 480, this.onInitCompleted);
        }, function () { }, function (workbench) { // End
            stopEaselJSBench();
        });

    var easelJSSpritesBench_1000_NoShadow_640_480 = new POTATOES.GamingBench.Bench("Drawing 1000 sprites (640x480)", "http://blogs.msdn.com/davrous",
        function (workbench) { // Init
            init(workbench, 1000, false, 640, 480, this.onInitCompleted);
        }, function () { }, function (workbench) { // End
            stopEaselJSBench();
        });

    var easelJSSpritesBench_5000_NoShadow_640_480 = new POTATOES.GamingBench.Bench("Drawing 5000 sprites (640x480)", "http://blogs.msdn.com/davrous",
        function (workbench) { // Init
            init(workbench, 5000, false, 640, 480, this.onInitCompleted);
        }, function () { }, function (workbench) { // End
            stopEaselJSBench();
        });

    //POTATOES.GamingBench.registerBench(easelJSSpritesBench_200_NoShadow_640_480);
    //POTATOES.GamingBench.registerBench(easelJSSpritesBench_200_Shadows_640_480);
    POTATOES.GamingBench.registerBench(easelJSSpritesBench_1000_NoShadow_640_480);
    //POTATOES.GamingBench.registerBench(easelJSSpritesBench_5000_NoShadow_640_480);
})();

